@@ -11,7 +11,9 @@
                       withgaps = TRUE,
                       simplifyD = TRUE,
                       DistrCollapse = TRUE,
-                      withSweave = FALSE
+                      withSweave = FALSE,
+                      ## new Items after mail by Jacob van Etter, 27-02-09
+                      DistrCollapse.Unique.Warn = FALSE
                       )
 distroptions <- function(...) {
   if (nargs() == 0) return(.distroptions)
