@@ -52,43 +52,62 @@ setAs("numeric","Integer",function(from) new("Integer",as.integer(from)))
 #------------------------------------------------------------------------------
 
 .fM <- function(x,f){
-   xo <- x
+   owarn <- getOption("warn")
+   options("warn"=-1)
+   on.exit(options("warn"=owarn))
+   if(!.inArgs("log.p", f))
+      f0 <- f
+   else f0 <- function(x) f(log(x), log.p = TRUE)   
+   xo <-  x
    x1 <- (1+xo)/2
    i <- 1
    while( i < 30)
    { i <- i+1
-     while(f(x1) < Inf)
+     while(!is.na(f1 <- f0(x1)) && f1 < Inf)
         {xo <- x1
          x1 <- (1+x1)/2
          }
      x1 <- (x1+xo)/2}
-   xo}
+   log(xo)}
    
 .fM2 <- function(x,f){
+   owarn <- getOption("warn")
+   options("warn"=-1)
+   on.exit(options("warn"=owarn))
+   if(!.inArgs("log.p", f))
+      f0 <- function(x) f(x, lower.tail = FALSE)
+   else f0 <- function(x) f(log(x), lower.tail = FALSE, log.p = TRUE)   
    xo <- x
    x1 <- xo/2
    i <- 1
    while( i < 30)
    { i <- i+1
-     while(f(x1, lower.tail = FALSE) < Inf)
+     while(!is.na(f1 <- f0(x1)) && f1 < Inf)
         {xo <- x1
          x1 <- x1/2
          }
-     x1 <- (x1+xo)/2}
-   xo}
+     x1 <- (x1+xo)/2}   
+   log(xo)}
 
 .fm <- function(x,f){
+   owarn <- getOption("warn")
+   options("warn"=-1)
+   on.exit(options("warn"=owarn))
+   if(!.inArgs("log.p", f))
+      f0 <- f
+   else f0 <- function(x) f(log(x), log.p = TRUE)   
+   
    xo <- x
    x1 <- xo/2
    i <- 1
    while( i < 30)
    { i <- i+1
-     while(f(x1) > -Inf)
+     while(!is.na(f1 <- f0(x1)) && f1 > -Inf)
         {xo <- x1
          x1 <- x1/2
          }
      x1 <- (x1+xo)/2}
-   xo}
+   log(xo)}
    
 #------------------------------------------------------------------------------
 # .presubs : for titles etc
@@ -177,6 +196,7 @@ return(outC)
    x <- seq(from = lower, to = upper, by = h)
    if(TRUE){#.notwithLArg(D)){
       return(diff(p(D)(x)))
+#      return((diff(p(D)(x))+diff(rev(p(D)(x,lower=FALSE))))/2)
    }else{
       M <- q(D)(0.5);   L <- length(x)
       x.l <- x [ x <= M ];  x.u <- x [ x >= M ]
@@ -386,7 +406,8 @@ return(outC)
                               r = rnew, d = dnew, p = pnew,
                               q = qnew, support = supportnew,
                               a = e1@a, b = e1@b + e2, X0 = e1@X0,
-                             .withSim = FALSE, .withArith = TRUE)
+                             .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
                 rm(supportnew)
 
             }else if (Dclass == "DiscreteDistribution"){
@@ -394,20 +415,23 @@ return(outC)
                               r = rnew, d = dnew, p = pnew,
                               q = qnew, support = supportnew,
                               a = 1, b = e2, X0 = e1,
-                             .withSim = FALSE, .withArith = TRUE)
+                             .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
                 rm(supportnew)
 
             }else if (Dclass == "AffLinAbscontDistribution"){
                 object <- new("AffLinAbscontDistribution", 
                               r = rnew, d = dnew, p = pnew, q = qnew, 
                               gaps = gapsnew, a = e1@a, b = e1@b + e2, 
-                              X0 = e1@X0, .withSim = FALSE, .withArith = TRUE)
+                              X0 = e1@X0, .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
 
             }else if (Dclass == "AbscontDistribution"){  
                 object <- new("AffLinAbscontDistribution", r = rnew, 
                               d = dnew, p = pnew, q = qnew, gaps = gapsnew, 
                               a = 1, b = e2, X0 = e1, .withSim = FALSE, 
-                              .withArith = TRUE)
+                              .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
             }
             rm(pnew, qnew, dnew, rnew)
             object
@@ -434,7 +458,10 @@ return(outC)
                  coR <- substitute({
                              o.warn <- getOption("warn"); options(warn = -1)
                              on.exit(options(warn=o.warn))
-                             d0 <- object@d(x = q / e2C)
+                             #
+                             x0 <- .setEqual(q / e2C, support(object))
+                             d0 <- object@d(x = x0)*(x0 %in% support(object))
+                             #
                              options(warn = o.warn)
                              if (!lower.tail) d0 <- -d0
                              p0 <- p0 + d0},
@@ -452,7 +479,8 @@ return(outC)
                  object <- new(Dclass, r = rnew, d = dnew, p = pnew,
                                q = qnew, support = supportnew,
                                a = e1@a * e2, b = e2 * e1@b, X0 = e1@X0,                              
-                              .withSim = FALSE, .withArith = TRUE)
+                              .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
                  rm(supportnew)
 
             }else if (Dclass == "DiscreteDistribution"){
@@ -480,7 +508,8 @@ return(outC)
                  object <- new("AffLinDiscreteDistribution", r = rnew, d = dnew, 
                                p = pnew, q = qnew, support = supportnew,
                                a = e2, b = 0, X0 = e1,                              
-                              .withSim = FALSE, .withArith = TRUE)
+                              .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
                  rm(supportnew)
 
             }else if (Dclass == "AffLinAbscontDistribution"){
@@ -501,7 +530,8 @@ return(outC)
                                 sign = e2>0)
                  object <- new(Dclass, r = rnew, d = dnew, p = pnew, q = qnew, 
                                gaps = gapsnew, a = e1@a * e2, b = e2 * e1@b, 
-                               X0 = e1@X0, .withSim = FALSE, .withArith = TRUE)
+                               X0 = e1@X0, .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
  
             }else if (Dclass == "AbscontDistribution"){
                  if(is.null(e1@gaps)) 
@@ -522,7 +552,8 @@ return(outC)
                  object <- new("AffLinAbscontDistribution", r = rnew, d = dnew, 
                                p = pnew, q = qnew, gaps = gapsnew, a = e2, 
                                b = 0, X0 = e1, .withSim = FALSE, 
-                               .withArith = TRUE)
+                               .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
             }
             rm(pnew, qnew, dnew, rnew)
             object
@@ -804,7 +835,8 @@ return(f)
             object <- new("DiscreteDistribution",
                            r = rnew, d = dnew, p = pnew,
                            q = qnew, support = supportnew,
-                          .withSim = FALSE, .withArith = TRUE)
+                          .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
             rm(supportnew)
             rm(pnew, qnew, dnew, rnew)
             object
@@ -845,7 +877,8 @@ return(f)
             object <- AbscontDistribution(
                            r = rnew, d = dnew, p = pnew,
                            q = qnew, gaps = gapsnew,
-                          .withSim = FALSE, .withArith = TRUE)
+                          .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
             rm(gapsnew)
             rm(pnew, qnew, dnew, rnew)
             object
@@ -867,7 +900,8 @@ return(f)
             object <- new("DiscreteDistribution",
                            r = rnew, d = dnew, p = pnew,
                            q = qnew, support = supportnew,
-                          .withSim = FALSE, .withArith = TRUE)
+                          .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
             rm(supportnew)
             rm(pnew, qnew, dnew, rnew)
             object
@@ -889,8 +923,9 @@ return(f)
             object <- AbscontDistribution(
                            r = rnew, d = dnew, p = pnew,
                            q = qnew, gaps = gapsnew,
-                          .withSim = FALSE, .withArith = TRUE)
-            rm(gapsnew)
+                          .withSim = FALSE, .withArith = TRUE,
+                    .logExact = .logExact(e1), .lowerExact = .lowerExact(e1))
+            if(exists("gapsnew")) rm(gapsnew)
             rm(pnew, qnew, dnew, rnew)
             object
           }
@@ -961,6 +996,83 @@ return(function(q, lower.tail = TRUE, log.p = FALSE){
 })
 }
 
+#------------------------------------------------------------------------------
+# modify slot q for AbscontDistribution if there are gaps
+#------------------------------------------------------------------------------
+.modifyqgaps <- function(pfun, qfun, gaps, leftright = "left"){
+  p.gaps <- pfun(gaps[,1]) 
+  p.gaps.l <- pfun(gaps[,1], lower.tail = FALSE)
+  dP <- deparse(body(qfun))
+  syC <- paste(sys.calls())
+  if(length(grep("q\\.r\\(",syC[length(syC)-2])) == 0)
+     if(length(grep("q0 <- qfun", dP[2]))>0) 
+        return(qfun)
+  if(pmatch(leftright, table = c("left","right"), nomatch = 1) == 1){
+     qnew <- function(p, lower.tail = TRUE, log.p = FALSE) {
+             q0 <- qfun(p, lower.tail = lower.tail, log.p = log.p)
+             i0 <- seq(length=length(p))
+             if(lower.tail){
+                if(log.p) p.gaps <- log(p.gaps)
+                for(i in 1:nrow(gaps)){
+                    i1 <- .isEqual(p,p.gaps[i])
+                    i2 <- i0[p<p.gaps[i]]
+                    i3 <- i0[p>p.gaps[i]]
+                    q0[i1] <- gaps[i,1]
+                    if(length(i2)>0)
+                       q0[i2] <- pmin(q0[i2],gaps[i,1])
+                    if(length(i3)>0)
+                       q0[i3] <- pmax(q0[i3],gaps[i,2])
+                }        
+             }else{
+                if(log.p) p.gaps.l <- log(p.gaps.l)
+                for(i in 1:nrow(gaps)){
+                    i1 <- .isEqual(p,p.gaps.l[i])
+                    i2 <- i0[p<p.gaps.l[i]]
+                    i3 <- i0[p>p.gaps.l[i]]
+                    q0[i1] <- gaps[i,2]
+                    if(length(i2)>0)
+                       q0[i2] <- pmax(q0[i2],gaps[i,2])    
+                    if(length(i3)>0)
+                       q0[i3] <- pmin(q0[i3],gaps[i,1])    
+                }        
+             }
+             return(q0)
+     }
+  }else{
+     qnew <- function(p, lower.tail = TRUE, log.p = FALSE) {
+             q0 <- qfun(p, lower.tail = lower.tail, log.p = log.p)
+             i0 <- seq(length=length(p))
+             if(lower.tail){
+                if(log.p) p.gaps <- log(p.gaps)
+                for(i in 1:nrow(gaps)){
+                    i1 <- .isEqual(p,p.gaps[i])
+                    i2 <- i0[p<p.gaps[i]]
+                    i3 <- i0[p>p.gaps[i]]
+                    q0[i1] <- gaps[i,2]
+                    if(length(i2)>0)
+                       q0[i2] <- pmin(q0[i2],gaps[i,1])
+                    if(length(i3)>0)
+                       q0[i3] <- pmax(q0[i3],gaps[i,2])
+                }        
+             }else{
+                if(log.p) p.gaps.l <- log(p.gaps.l)
+                for(i in 1:nrow(gaps)){
+                    i1 <- .isEqual(p,p.gaps.l[i])
+                    i2 <- i0[p<p.gaps.l[i]]
+                    i3 <- i0[p>p.gaps.l[i]]
+                    q0[i1] <- gaps[i,1]
+                    if(length(i2)>0)
+                       q0[i2] <- pmax(q0[i2],gaps[i,2])    
+                    if(length(i3)>0)
+                       q0[i3] <- pmin(q0[i3],gaps[i,1])    
+                }        
+             }
+             return(q0)
+     }
+  }
+  return(qnew)           
+}
+
 #------------------------------------------------------------------------------
 # issue warnings in show / print as to Arith or print
 #------------------------------------------------------------------------------
@@ -980,3 +1092,39 @@ return(function(q, lower.tail = TRUE, log.p = FALSE){
        }
     return(list(msgA=c(msgA1,msgA2), msgS = c(msgS1,msgS2)))  
     }
+
+#------------------------------------------------------------------------------
+# fill a list acc. recycling rules
+#------------------------------------------------------------------------------
+.fillList <- function(list0, len = length(list0)){
+            if(len == length(list0)) 
+               return(list0)
+            i <- 0
+            ll0 <- length(list0)
+            li0 <- vector("list",len)
+            if(ll0)
+            while(i < len){
+               j <- 1 + ( i %% ll0)
+               i <- i + 1
+               li0[[i]] <- list0[[j]]
+            }
+           return(li0)
+}
+
+#------------------------------------------------------------------------------
+# .DistributionAggregate by Jacob van Etten, jacobvanetten@yahoo.com
+# on a mail on Feb 27th 2009
+#------------------------------------------------------------------------------
+.DistrCollapse <- function(support, prob, 
+                              eps = getdistrOption("DistrResolution")){
+    supp <- support
+    prob <- as.vector(prob)
+    suppIncr <- diff(c(supp[1]-2*eps,supp)) < eps
+    groups <- cumsum(!suppIncr)
+    prob <- as.vector(tapply(prob, groups, sum))
+    supp <- as.vector(tapply(supp, groups, quantile, probs = 0.5, type = 1)) 
+           ### in order to get a "support member" take the leftmost median
+    return(list(supp = supp, prob = prob))
+#    newDistribution <- DiscreteDistribution(supp=supp,prob=prob)
+#    return(newDistribution)
+}
